@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-gst.c,v 1.108 2004/11/19 12:09:29 rbultje Exp $
+ * $Id: bacon-video-widget-gst.c,v 1.119 2004/12/24 17:41:34 hadess Exp $
  *
  */
 
@@ -117,7 +117,7 @@ struct BaconVideoWidgetPrivate
   /* Visual effects */
   GList *vis_plugins_list;
   gboolean show_vfx;
-  gboolean using_vfx;
+  VisualsQuality visq;
   GstElement *vis_element;
 
   /* Other stuff */
@@ -143,7 +143,8 @@ enum {
   ASYNC_VIDEO_SIZE,
   ASYNC_ERROR,
   ASYNC_FOUND_TAG,
-  ASYNC_EOS
+  ASYNC_EOS,
+  ASYNC_BUFFERING
 };
 
 typedef struct _BVWSignal BVWSignal;
@@ -169,6 +170,10 @@ struct _BVWSignal
       GstElement *source;
       GstTagList *tag_list;
     } found_tag;
+    struct
+    {
+      gint percent;
+    } buffering;
   } signal_data;
 };
 
@@ -190,7 +195,9 @@ static int bvw_table_signals[LAST_SIGNAL] = { 0 };
 static void
 get_media_size (BaconVideoWidget *bvw, gint *width, gint *height)
 {
-  if (GST_STATE (bvw->priv->play) >= GST_STATE_PAUSED) {
+  if (GST_STATE (bvw->priv->play) >= GST_STATE_PAUSED &&
+      (bvw->priv->media_has_video ||
+       (bvw->priv->show_vfx && bvw->priv->vis_element))) {
     *width = bvw->priv->video_width;
     *height = bvw->priv->video_height;
   } else {
@@ -281,7 +288,9 @@ bacon_video_widget_expose_event (GtkWidget *widget, GdkEventExpose *event)
 		      widget->allocation.width, widget->allocation.height);
   gst_x_overlay_set_xwindow_id (bvw->priv->xoverlay, window);
 
-  if (GST_STATE (bvw->priv->play) >= GST_STATE_PAUSED) {
+  if (GST_STATE (bvw->priv->play) >= GST_STATE_PAUSED &&
+      (bvw->priv->media_has_video ||
+       (bvw->priv->vis_element && bvw->priv->show_vfx))) {
     gst_x_overlay_expose (bvw->priv->xoverlay);
   } else if (bvw->priv->logo_pixbuf != NULL) {
     /* draw logo here */
@@ -353,7 +362,9 @@ bacon_video_widget_size_allocate (GtkWidget * widget,
                             allocation->width, allocation->height);
 
     /* resize video_window */
-    if (GST_STATE (bvw->priv->play) >= GST_STATE_PAUSED) {
+    if (GST_STATE (bvw->priv->play) >= GST_STATE_PAUSED &&
+        (bvw->priv->media_has_video ||
+         (bvw->priv->show_vfx && bvw->priv->vis_element))) {
       width = bvw->priv->video_width;
       height = bvw->priv->video_height;
     } else if (bvw->priv->logo_pixbuf) {
@@ -632,6 +643,12 @@ bacon_video_widget_signal_idler (BaconVideoWidget *bvw)
           g_signal_emit (G_OBJECT (bvw), bvw_table_signals[SIGNAL_EOS], 0, NULL);
           break;
         }
+      case ASYNC_BUFFERING:
+        {
+          g_signal_emit (G_OBJECT (bvw), bvw_table_signals[SIGNAL_BUFFERING],
+			 0, signal->signal_data.buffering.percent);
+          break;
+        }
       default:
         break;
     }
@@ -696,6 +713,24 @@ got_eos (GstElement * play, BaconVideoWidget * bvw)
   g_idle_add ((GSourceFunc) bacon_video_widget_signal_idler, bvw);
 }
 
+static void
+got_buffering (GstElement * play, gint percentage,
+	       BaconVideoWidget * bvw)
+{
+  BVWSignal *signal;
+
+  g_return_if_fail (bvw != NULL);
+  g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
+
+  signal = g_new0 (BVWSignal, 1);
+  signal->signal_id = ASYNC_BUFFERING;
+  signal->signal_data.buffering.percent = percentage;
+
+  g_async_queue_push (bvw->priv->queue, signal);
+
+  g_idle_add ((GSourceFunc) bacon_video_widget_signal_idler, bvw);
+}
+
 static void
 got_stream_length (GstElement * play, gint64 length_nanos,
                    BaconVideoWidget * bvw)
@@ -740,6 +775,28 @@ got_time_tick (GstElement * play, gint64 time_nanos, BaconVideoWidget * bvw)
                  seekable);
 }
 
+static void
+got_source (GObject    *play,
+	    GParamSpec *pspec,
+	    BaconVideoWidget *bvw)
+{
+  GObject *source = NULL;
+  GObjectClass *klass;
+
+  if (!bvw->priv->media_device)
+    return;
+
+  g_object_get (play, "source", &source, NULL);
+  if (!source)
+    return;
+
+  klass = G_OBJECT_GET_CLASS (source);
+  if (!g_object_class_find_property (klass, "device"))
+    return;
+
+  g_object_set (source, "device", bvw->priv->media_device, NULL);
+}
+
 static gboolean
 cb_iterate (BaconVideoWidget *bvw)
 {
@@ -825,6 +882,7 @@ state_change (GstElement *play, GstElementState old_state,
   if (old_state <= GST_STATE_READY &&
       new_state >= GST_STATE_PAUSED) {
     GList *streaminfo = NULL;
+    GstPad *videopad = NULL;
 
     g_object_get (G_OBJECT (bvw->priv->play), "stream-info",
 		  &streaminfo, NULL);
@@ -839,25 +897,36 @@ state_change (GstElement *play, GstElementState old_state,
       val = g_enum_get_value (G_PARAM_SPEC_ENUM (pspec)->enum_class, type);
 
       if (strstr (val->value_name, "AUDIO")) {
-        bvw->priv->media_has_audio = TRUE;
-      } else if (strstr (val->value_name, "VIDEO")) {
-        GstPad *pad = NULL;
-
-        g_object_get (info, "object", &pad, NULL);
-        if (pad != NULL && GST_PAD_REALIZE (pad) != NULL) {
-          g_assert (GST_IS_PAD (pad));
-
-          /* handle explicit caps as well - they're set later */
-          if (GST_PAD_REALIZE (pad)->link != NULL && GST_PAD_CAPS (pad))
-            caps_set (G_OBJECT (pad), NULL, bvw);
-          else
-            g_signal_connect (pad, "notify::caps",
-                G_CALLBACK (caps_set), bvw);
+        if (!bvw->priv->media_has_audio) {
+          bvw->priv->media_has_audio = TRUE;
+          if (!bvw->priv->media_has_video &&
+              bvw->priv->show_vfx && bvw->priv->vis_element) {
+            videopad = gst_element_get_pad (bvw->priv->vis_element, "src");
+          }
         }
+      } else if (strstr (val->value_name, "VIDEO")) {
+        if (!bvw->priv->media_has_video) {
+          bvw->priv->media_has_video = TRUE;
 
-        bvw->priv->media_has_video = TRUE;
+          videopad = NULL;
+          g_object_get (info, "object", &videopad, NULL);
+          /* hack for 0.8.5 */
+          if (!videopad)
+            g_object_get (info, "pad", &videopad, NULL);
+        }
       }
     }
+
+    if (videopad) {
+      GstPad *real = (GstPad *) GST_PAD_REALIZE (videopad);
+
+      /* handle explicit caps as well - they're set later */
+      if (((GstRealPad *) real)->link != NULL && GST_PAD_CAPS (real))
+        caps_set (G_OBJECT (real), NULL, bvw);
+      else
+        g_signal_connect (real, "notify::caps",
+            G_CALLBACK (caps_set), bvw);
+    }
   } else if (new_state <= GST_STATE_READY &&
              old_state >= GST_STATE_PAUSED) {
     bvw->priv->media_has_video = FALSE;
@@ -1222,6 +1291,51 @@ bacon_video_widget_dvd_event (BaconVideoWidget * bvw,
   g_return_if_fail (bvw != NULL);
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
   g_return_if_fail (GST_IS_ELEMENT (bvw->priv->play));
+
+  switch (type) {
+    case BVW_DVD_ROOT_MENU:
+    case BVW_DVD_TITLE_MENU:
+    case BVW_DVD_SUBPICTURE_MENU:
+    case BVW_DVD_AUDIO_MENU:
+    case BVW_DVD_ANGLE_MENU:
+    case BVW_DVD_CHAPTER_MENU:
+      /* FIXME */
+      break;
+    case BVW_DVD_NEXT_CHAPTER:
+    case BVW_DVD_PREV_CHAPTER:
+    case BVW_DVD_NEXT_TITLE:
+    case BVW_DVD_PREV_TITLE:
+    case BVW_DVD_NEXT_ANGLE:
+    case BVW_DVD_PREV_ANGLE: {
+      GstFormat fmt;
+      gint64 val;
+      gint dir;
+
+      if (type == BVW_DVD_NEXT_CHAPTER ||
+          type == BVW_DVD_NEXT_TITLE ||
+          type == BVW_DVD_NEXT_ANGLE)
+        dir = 1;
+      else
+        dir = -1;
+
+      if (type == BVW_DVD_NEXT_CHAPTER || type == BVW_DVD_PREV_CHAPTER)
+        fmt = gst_format_get_by_nick ("chapter");
+      else if (type == BVW_DVD_NEXT_TITLE || type == BVW_DVD_PREV_TITLE)
+        fmt == gst_format_get_by_nick ("title");
+      else
+        fmt = gst_format_get_by_nick ("angle");
+
+      if (gst_element_query (bvw->priv->play,
+          GST_QUERY_POSITION, &fmt, &val)) {
+        val += dir;
+        gst_element_seek (bvw->priv->play,
+            fmt | GST_SEEK_METHOD_SET | GST_SEEK_FLAG_FLUSH, val);
+      }
+      break;
+    }
+    default:
+      break;
+  }
 }
 
 void
@@ -1377,12 +1491,92 @@ bacon_video_widget_set_media_device (BaconVideoWidget * bvw, const char *path)
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
   g_return_if_fail (GST_IS_ELEMENT (bvw->priv->play));
   
-  if (bvw->priv->media_device)
-    g_free (bvw->priv->media_device);
-  
+  g_free (bvw->priv->media_device);
   bvw->priv->media_device = g_strdup (path);
 }
 
+static GstCaps *
+fixate_visualization (GstPad *pad, const GstCaps *in_caps,
+		      BaconVideoWidget *bvw)
+{
+  GstCaps *caps;
+  GstStructure *s;
+  int fps, w, h;
+
+  switch (bvw->priv->visq)
+    {
+    case VISUAL_SMALL:
+      fps = 10;
+      w = 200;
+      h = 150;
+      break;
+    case VISUAL_NORMAL:
+      fps = 20;
+      w = 320;
+      h = 240;
+      break;
+    case VISUAL_LARGE:
+      fps = 25;
+      w = 640;
+      h = 480;
+      break;
+    case VISUAL_EXTRA_LARGE:
+      fps = 30;
+      w = 800;
+      h = 600;
+      break;
+    default:
+      /* shut up warnings */
+      fps = w = h = 0;
+      g_assert_not_reached ();
+  }
+
+  /* simple please */
+  if (gst_caps_get_size (in_caps) > 1)
+    return NULL;
+
+  /* fixate */
+  caps = gst_caps_copy (in_caps);
+  s = gst_caps_get_structure (caps, 0);
+  if (gst_caps_structure_fixate_field_nearest_int (s, "width", w) ||
+      gst_caps_structure_fixate_field_nearest_int (s, "height", h) ||
+      gst_caps_structure_fixate_field_nearest_double (s, "framerate", fps))
+    return caps;
+
+  gst_caps_free (caps);
+  return NULL;
+}
+
+static void
+setup_vis (BaconVideoWidget * bvw)
+{
+  const GstElement *vis;
+  gint rel = -1, mic = 0;
+
+  /* only if >= 0.8.6.1, else it'll hang after one song (refcount bug) */
+  if (rel == -1) {
+    GstPlugin *p;
+
+    p = gst_registry_pool_find_plugin ("playbin");
+    g_assert (p); /* otherwise we can't be here */
+    if (sscanf (p->desc.version, "%*d.%*d.%d.%d", &rel, &mic) < 1)
+      g_assert_not_reached ();
+
+    /* only if >= 0.8.6.1 */
+    if (rel < 6 || (rel == 6 && mic < 1)) {
+      g_warning ("Visualization disabled because your gst-plugins is too old (%d.%d.%d.%d)", 0, 8, rel, mic);
+      return;
+    }
+  }
+
+  if (bvw->priv->show_vfx && bvw->priv->vis_element)
+    vis = bvw->priv->vis_element;
+  else
+    vis = NULL;
+
+  g_object_set (G_OBJECT (bvw->priv->play), "vis-plugin", vis, NULL);
+}
+
 gboolean
 bacon_video_widget_set_show_visuals (BaconVideoWidget * bvw,
 				     gboolean show_visuals)
@@ -1391,20 +1585,11 @@ bacon_video_widget_set_show_visuals (BaconVideoWidget * bvw,
   g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
   g_return_val_if_fail (GST_IS_ELEMENT (bvw->priv->play), FALSE);
 
-  return FALSE;
-#if 0
-  if (!bvw->priv->media_has_video)
-    {
-      if (!show_visuals)
-	gst_video_widget_set_logo_focus (bvw->priv->vw, TRUE);
-      else
-	gst_video_widget_set_logo_focus (bvw->priv->vw, FALSE);
+  bvw->priv->show_vfx = show_visuals;
 
-      gst_play_connect_visualization (bvw->priv->play, show_visuals);
-    }
+  setup_vis (bvw);
 
   return TRUE;
-#endif
 }
 
 GList *
@@ -1475,82 +1660,43 @@ bacon_video_widget_get_visuals_list (BaconVideoWidget * bvw)
 gboolean
 bacon_video_widget_set_visuals (BaconVideoWidget * bvw, const char *name)
 {
-  /*gboolean paused = FALSE;*/
+  GstElement *old_vis = bvw->priv->vis_element;
 
   g_return_val_if_fail (bvw != NULL, FALSE);
   g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
   g_return_val_if_fail (GST_IS_ELEMENT (bvw->priv->play), FALSE);
 
-  return FALSE;
-#if 0
-  if (GST_STATE (GST_ELEMENT (bvw->priv->play)) == GST_STATE_PLAYING)
-    {
-      gst_element_set_state (GST_ELEMENT (bvw->priv->play), GST_STATE_PAUSED);
-      paused = TRUE;
-    }
-
   bvw->priv->vis_element = gst_element_factory_make (name,
                                                      "vis_plugin_element");
-  if (GST_IS_ELEMENT (bvw->priv->vis_element))
-    {
-      gst_play_set_visualization (bvw->priv->play, bvw->priv->vis_element);
-      if (paused)
-	{
-	  gst_play_seek_to_time (bvw->priv->play,
-				 bvw->priv->current_time_nanos);
-	  gst_element_set_state (GST_ELEMENT (bvw->priv->play),
-                                 GST_STATE_PLAYING);
-	}
+  if (!bvw->priv->vis_element) {
+    bvw->priv->vis_element = old_vis;
+    return FALSE;
+  }
 
-      return FALSE;
-    }
-  else
-    {
-      return FALSE;
-    }
-#endif
+  g_signal_connect (gst_element_get_pad (bvw->priv->vis_element, "src"),
+		    "fixate", G_CALLBACK (fixate_visualization), bvw);
+
+  setup_vis (bvw);
+  if (old_vis) {
+    gst_object_unref (GST_OBJECT (old_vis));
+  }
+
+  return TRUE;
 }
 
 void
 bacon_video_widget_set_visuals_quality (BaconVideoWidget * bvw,
 					VisualsQuality quality)
 {
-  int fps, w, h;
   g_return_if_fail (bvw != NULL);
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
   g_return_if_fail (GST_IS_ELEMENT (bvw->priv->play));
 
-  if (bvw->priv->vis_element == NULL)
+  if (bvw->priv->visq == quality)
     return;
+  bvw->priv->visq = quality;
 
-  switch (quality)
-    {
-    case VISUAL_SMALL:
-      fps = 15;
-      w = 320;
-      h = 240;
-      break;
-    case VISUAL_NORMAL:
-      fps = 25;
-      w = 320;
-      h = 240;
-      break;
-    case VISUAL_LARGE:
-      fps = 25;
-      w = 640;
-      h = 480;
-      break;
-    case VISUAL_EXTRA_LARGE:
-      fps = 30;
-      w = 800;
-      h = 600;
-      break;
-    default:
-      /* shut up warnings */
-      fps = w = h = 0;
-      g_assert_not_reached ();
-  }
-
+  //gst_pad_renegotiate (gst_element_get_pad (bvw->priv->vis_element, "src"));
 }
 
 gboolean
@@ -1620,6 +1766,31 @@ bacon_video_widget_set_scale_ratio (BaconVideoWidget * bvw, gfloat ratio)
   totem_widget_set_preferred_size (GTK_WIDGET (bvw), w, h);
 }
 
+gboolean
+bacon_video_widget_can_set_zoom (BaconVideoWidget *bvw)
+{
+  return FALSE;
+}
+
+void
+bacon_video_widget_set_zoom (BaconVideoWidget *bvw,
+			     int               zoom)
+{
+  g_return_if_fail (bvw != NULL);
+  g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
+
+  /* implement me */
+}
+
+int
+bacon_video_widget_get_zoom (BaconVideoWidget *bvw)
+{
+  g_return_val_if_fail (bvw != NULL, 100);
+  g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), 100);
+
+  return 100;
+}
+
 int
 bacon_video_widget_get_video_property (BaconVideoWidget *bvw,
                                        BaconVideoWidgetVideoProperty type)
@@ -1799,9 +1970,9 @@ bacon_video_widget_can_play (BaconVideoWidget * bvw, MediaType type)
   switch (type) {
     case MEDIA_TYPE_CDDA:
     case MEDIA_TYPE_VCD:
+    case MEDIA_TYPE_DVD:
       res = TRUE;
       break;
-    case MEDIA_TYPE_DVD:
     default:
       res = FALSE;
       break;
@@ -1869,9 +2040,16 @@ bacon_video_widget_get_metadata_string (BaconVideoWidget * bvw,
       res = gst_tag_list_get_string (bvw->priv->tagcache,
 				     GST_TAG_ARTIST, &string);
       break;
-    case BVW_INFO_YEAR:
+    case BVW_INFO_YEAR: {
+      guint year;
+      res = gst_tag_list_get_uint (bvw->priv->tagcache,
+				   GST_TAG_DATE, &year);
+      string = g_strdup_printf ("%u", year);
+      break;
+    }
+    case BVW_INFO_ALBUM:
       res = gst_tag_list_get_string (bvw->priv->tagcache,
-				     GST_TAG_DATE, &string);
+		      		     GST_TAG_ALBUM, &string);
       break;
     case BVW_INFO_VIDEO_CODEC:
       res = gst_tag_list_get_string (bvw->priv->tagcache,
@@ -2006,6 +2184,7 @@ bacon_video_widget_get_metadata (BaconVideoWidget * bvw,
     case BVW_INFO_TITLE:
     case BVW_INFO_ARTIST:
     case BVW_INFO_YEAR:
+    case BVW_INFO_ALBUM:
     case BVW_INFO_VIDEO_CODEC:
     case BVW_INFO_AUDIO_CODEC:
       bacon_video_widget_get_metadata_string (bvw, type, value);
@@ -2080,6 +2259,7 @@ bacon_video_widget_get_current_frame (BaconVideoWidget * bvw)
       "stream-info", &streaminfo, NULL);
   for (; streaminfo != NULL; streaminfo = streaminfo->next) {
     GObject *info = streaminfo->data;
+    GstPad *real;
     gint type;
     GParamSpec *pspec;
     GEnumValue *val;
@@ -2087,13 +2267,17 @@ bacon_video_widget_get_current_frame (BaconVideoWidget * bvw)
     g_object_get (info, "type", &type, NULL);
     pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (info), "type");
     val = g_enum_get_value (G_PARAM_SPEC_ENUM (pspec)->enum_class, type);
-                                                                                
+
     if (strstr (val->value_name, "VIDEO")) {
-      GstPad *pad;
+      GstPad *pad = NULL;
 
       g_object_get (info, "object", &pad, NULL);
+      /* hack for 0.8.5 */
+      if (!pad)
+        g_object_get (info, "pad", &pad, NULL);
       g_assert (GST_IS_PAD (pad));
-      from = gst_caps_copy (GST_PAD_CAPS (pad));
+      real = (GstPad *) GST_PAD_REALIZE (pad);
+      from = gst_caps_copy (GST_PAD_CAPS (real));
     }
   }
   if (!from)
@@ -2167,7 +2351,7 @@ out_error (GstElement *el, GstElement *src,
 
 GtkWidget *
 bacon_video_widget_new (int width, int height,
-			gboolean null_out, GError ** err)
+			BvwUseType type, GError ** err)
 {
   BaconVideoWidget *bvw;
   GstElement *audio_sink = NULL, *video_sink = NULL;
@@ -2192,21 +2376,28 @@ bacon_video_widget_new (int width, int height,
   bvw->priv->logo_mode = TRUE;
   bvw->priv->auto_resize = TRUE;
 
-  if (!null_out) {
+  if (type == BVW_USE_TYPE_VIDEO || type == BVW_USE_TYPE_AUDIO) {
     audio_sink = gst_gconf_get_default_audio_sink ();
     if (!GST_IS_ELEMENT (audio_sink))
       {
         g_message ("failed to render default audio sink from gconf");
         return NULL;
       }
+  }
+
+  if (type == BVW_USE_TYPE_VIDEO) {
     video_sink = gst_gconf_get_default_video_sink ();
     if (!GST_IS_ELEMENT (video_sink))
       {
         g_message ("failed to render default video sink from gconf");
         return NULL;
       }
-  } else {
+  }
+
+  if (video_sink == NULL) {
     video_sink = gst_element_factory_make ("fakesink", "fakevideosink");
+  }
+  if (audio_sink == NULL) {
     audio_sink = gst_element_factory_make ("fakesink", "fakeaudiosink");
   }
 
@@ -2246,9 +2437,13 @@ bacon_video_widget_new (int width, int height,
 		    G_CALLBACK (got_found_tag), (gpointer) bvw);
   g_signal_connect (G_OBJECT (bvw->priv->play), "error",
 		    G_CALLBACK (got_error), (gpointer) bvw);
+  g_signal_connect (G_OBJECT (bvw->priv->play), "buffering",
+		    G_CALLBACK (got_buffering), (gpointer) bvw);
+  g_signal_connect (G_OBJECT (bvw->priv->play), "notify::source",
+		    G_CALLBACK (got_source), (gpointer) bvw);
 
   /* We try to get an element supporting XOverlay interface */
-  if (!null_out && GST_IS_BIN (bvw->priv->play)) {
+  if (type == BVW_USE_TYPE_VIDEO && GST_IS_BIN (bvw->priv->play)) {
     GstElement *element = NULL;
     GList *elements = NULL, *l_elements = NULL;
     
