@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.139 2004/11/08 21:22:30 hadess Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.157 2005/01/03 12:50:18 hadess Exp $
  *
  * the xine engine in a widget - implementation
  */
@@ -41,6 +41,7 @@
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
+#include <glib/gi18n.h>
 /* xine */
 #include <xine.h>
 
@@ -51,14 +52,6 @@
 #include "video-utils.h"
 #include "bacon-resize.h"
 
-#include <libintl.h>
-#define _(String) gettext (String)
-#ifdef gettext_noop
-#   define N_(String) gettext_noop (String)
-#else
-#   define N_(String) (String)
-#endif
-
 #define DEFAULT_HEIGHT 315
 #define DEFAULT_WIDTH 420
 #define CONFIG_FILE ".gnome2"G_DIR_SEPARATOR_S"totem_config"
@@ -135,8 +128,8 @@ struct BaconVideoWidgetPrivate {
 
 	/* Configuration */
 	GConfClient *gc;
-	gboolean null_out;
 	char *mrl;
+	BvwUseType type;
 
 	/* X stuff */
 	Display *display;
@@ -161,7 +154,7 @@ struct BaconVideoWidgetPrivate {
 	gboolean can_dvd, can_vcd, can_cdda;
 	gboolean logo_mode;
 	guint tick_id;
-	gboolean have_xrandr;
+	gboolean have_xvidmode;
 	gboolean auto_resize;
 	int volume;
 	TvOutType tvout;
@@ -396,14 +389,13 @@ bacon_video_widget_init (BaconVideoWidget *bvw)
 	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
 
 	GTK_WIDGET_SET_FLAGS (GTK_WIDGET (bvw), GTK_CAN_FOCUS);
-	/* We work around white artifacts by using a double-buffered widget */
-	/* FIXME, this work-around breaks exposes */
 	GTK_WIDGET_UNSET_FLAGS (GTK_WIDGET (bvw), GTK_DOUBLE_BUFFERED);
 
 	bvw->priv = g_new0 (BaconVideoWidgetPrivate, 1);
 	bvw->priv->xine = xine_new ();
 	bvw->priv->cursor_shown = TRUE;
 	bvw->priv->vis_name = g_strdup ("goom");
+	bvw->priv->volume = -1;
 
 	bvw->priv->init_width = 0;
 	bvw->priv->init_height = 0;
@@ -530,10 +522,6 @@ frame_output_cb (void *bvw_gen,
 			g_async_queue_push (bvw->priv->queue, data);
 			g_idle_add ((GSourceFunc)
 					bacon_video_widget_idle_signal, bvw);
-		} else if (bvw->priv->auto_resize != FALSE
-				&& bvw->priv->have_xrandr != FALSE
-				&& bvw->priv->fullscreen_mode != FALSE) {
-			bacon_resize (video_height, video_width);
 		}
 	}
 
@@ -624,9 +612,6 @@ load_audio_out_driver (BaconVideoWidget *bvw, GError **error)
 	xine_audio_port_t *ao_driver;
 	const char *audio_driver_id;
 
-	if (bvw->priv->null_out != FALSE)
-		return NULL;
-
 	audio_driver_id = xine_config_register_string (bvw->priv->xine,
 			"audio.driver", "auto", "audio driver to use",
 			NULL, 10, NULL, NULL);
@@ -716,13 +701,6 @@ setup_config (BaconVideoWidget *bvw)
 	entry.num_value = 0;
 	xine_config_update_entry (bvw->priv->xine, &entry);
 
-	/* The volume */
-	xine_config_register_range (bvw->priv->xine,
-			"misc.amp_level",
-			50, 0, 100, "amp volume level",
-			NULL, 10, NULL, NULL);
-	bvw->priv->volume = -1;
-
 	if (bvw->priv->gc == NULL)
 		return;
 
@@ -805,21 +783,11 @@ setup_config (BaconVideoWidget *bvw)
 	}
 }
 
-static void
-setup_config_video (BaconVideoWidget *bvw)
-{
-	xine_cfg_entry_t entry;
-
-	/* Remove the ALSA HW mixing */
-	bvw_config_helper_num (bvw->priv->xine, "audio.alsa_hw_mixer", 0, &entry);
-	entry.num_value = 0;
-	xine_config_update_entry (bvw->priv->xine, &entry);
-}
-
 static void
 setup_config_stream (BaconVideoWidget *bvw)
 {
-	int value, i;
+	GConfValue *confvalue;
+	int value, i, tmp;
 
 	if (bvw->priv->gc == NULL)
 		return;
@@ -827,15 +795,16 @@ setup_config_stream (BaconVideoWidget *bvw)
 	/* Setup brightness and contrast */
 	for (i = 0; i < 4; i++)
 	{
-		GError *error = NULL;
-		int tmp;
+		confvalue = gconf_client_get_without_default (bvw->priv->gc,
+				video_props_str[i], NULL);
 
-		value = gconf_client_get_int (bvw->priv->gc,
-				video_props_str[i], &error);
-
-		/* avoid black screens */
-		if (value == 0 && error != NULL)
+		if (confvalue != NULL)
+		{
+			value = gconf_value_get_int (confvalue);
+			gconf_value_free (confvalue);
+		} else {
 			value = 65535 / 2;
+		}
 
 		tmp = xine_get_param (bvw->priv->stream, video_props[i]);
 		if (value != tmp)
@@ -843,9 +812,6 @@ setup_config_stream (BaconVideoWidget *bvw)
 			xine_set_param (bvw->priv->stream,
 					video_props[i], value);
 		}
-
-		if (error != NULL)
-			g_error_free (error);
 	}
 }
 
@@ -1062,30 +1028,15 @@ bacon_video_widget_realize (GtkWidget *widget)
 	BaconVideoWidget *bvw;
 
 	bvw = BACON_VIDEO_WIDGET (widget);
-
-	if (bvw->priv->mrl != NULL) {
-		bacon_video_widget_close (bvw);
-		g_warning ("bvw->priv->mrl != NULL on open");
+	if (bvw->priv->type != BVW_USE_TYPE_VIDEO)
+	{
+		g_warning ("Use type isn't video but we realized the widget");
+		return;
 	}
 
 	/* set realized flag */
 	GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
 
-	/* Close the old stream */
-	xine_close (bvw->priv->stream);
-	xine_event_dispose_queue (bvw->priv->ev_queue);
-	xine_dispose (bvw->priv->stream);
-	bvw->priv->stream = NULL;
-
-	if (bvw->priv->vo_driver != NULL) {
-		xine_close_video_driver (bvw->priv->xine,
-				bvw->priv->vo_driver);
-	}
-	if (bvw->priv->ao_driver != NULL) {
-		xine_close_audio_driver (bvw->priv->xine,
-				bvw->priv->ao_driver);
-	}
-
 	/* Create the widget's window */
 	attr.x = widget->allocation.x;
 	attr.y = widget->allocation.y;
@@ -1126,7 +1077,7 @@ bacon_video_widget_realize (GtkWidget *widget)
 	bvw->priv->screen = DefaultScreen (bvw->priv->display);
 
 	bvw->priv->vo_driver = load_video_out_driver
-		(bvw, bvw->priv->null_out);
+		(bvw, FALSE);
 
 	if (bvw->priv->vo_driver == NULL)
 	{
@@ -1146,16 +1097,15 @@ bacon_video_widget_realize (GtkWidget *widget)
 
 	bvw->priv->ao_driver = load_audio_out_driver (bvw, NULL);
 
-	setup_config_video (bvw);
-
-	if (bvw->priv->null_out == FALSE && bvw->priv->ao_driver != NULL)
+	if (bvw->priv->type == BVW_USE_TYPE_VIDEO
+			&& bvw->priv->ao_driver != NULL)
 	{
 		bvw->priv->vis = xine_post_init (bvw->priv->xine,
 				bvw->priv->vis_name, 0,
 				&bvw->priv->ao_driver, &bvw->priv->vo_driver);
 	}
 
-	bvw->priv->have_xrandr = bacon_resize_init ();
+	bvw->priv->have_xvidmode = bacon_resize_init ();
 	bvw->priv->stream = xine_stream_new (bvw->priv->xine,
 			bvw->priv->ao_driver, bvw->priv->vo_driver);
 	setup_config_stream (bvw);
@@ -1336,10 +1286,24 @@ xine_event (void *user_data, const xine_event_t *event)
 	xine_ui_data_t *ui_data;
 	xine_progress_data_t *prg;
 	xine_mrl_reference_data_t *ref;
+	xine_spu_button_t *spubtn;
 	signal_data *data;
 
 	switch (event->type)
 	{
+	case XINE_EVENT_SPU_BUTTON:
+		spubtn = (xine_spu_button_t *) event->data;
+		if (spubtn->direction)
+		{
+			GdkCursor *cursor;
+			cursor = gdk_cursor_new (GDK_HAND2);
+			gdk_window_set_cursor
+				(bvw->priv->video_window, cursor);
+			gdk_cursor_unref (cursor);
+		} else {
+			gdk_window_set_cursor(bvw->priv->video_window, NULL);
+		}
+		break;
 	case XINE_EVENT_UI_PLAYBACK_FINISHED:
 		data = g_new0 (signal_data, 1);
 		data->signal = EOS_ASYNC;
@@ -1466,7 +1430,6 @@ bacon_video_widget_unrealize (GtkWidget *widget)
 {
 	BaconVideoWidget *bvw;
 	char *configfile;
-	xine_cfg_entry_t entry;
 	int speed;
 
 	g_return_if_fail (widget != NULL);
@@ -1484,13 +1447,11 @@ bacon_video_widget_unrealize (GtkWidget *widget)
 	xine_stop (bvw->priv->stream);
 	xine_close (bvw->priv->stream);
 
-	/* Put the current volume in the config system */
+	/* Save the current volume */
 	if (bacon_video_widget_can_set_volume (bvw) != FALSE)
 	{
-		xine_config_lookup_entry (bvw->priv->xine,
-				"misc.amp_level", &entry);
-		entry.num_value = bvw->priv->volume;
-		xine_config_update_entry (bvw->priv->xine, &entry);
+		gconf_client_set_int (bvw->priv->gc, GCONF_PREFIX"/volume",
+				bvw->priv->volume, NULL);
 	}
 
 	/* Kill the TV out */
@@ -1498,6 +1459,15 @@ bacon_video_widget_unrealize (GtkWidget *widget)
         nvtv_simple_exit();
 #endif
 
+
+/* FIXME missing stuff from libxine includes (<= 1-rc7) */
+#ifndef XINE_GUI_SEND_WILL_DESTROY_DRAWABLE
+#define XINE_GUI_SEND_WILL_DESTROY_DRAWABLE 9
+#endif
+	xine_port_send_gui_data (bvw->priv->vo_driver,
+			XINE_GUI_SEND_WILL_DESTROY_DRAWABLE,
+			(void*)bvw->priv->video_window);
+
 	/* Hide all windows */
 	if (GTK_WIDGET_MAPPED (widget))
 		gtk_widget_unmap (widget);
@@ -1551,10 +1521,9 @@ bacon_video_widget_error_quark (void)
 	return q;
 }
 
-
 GtkWidget *
 bacon_video_widget_new (int width, int height,
-		gboolean null_out, GError **error)
+		BvwUseType type, GError **error)
 {
 	BaconVideoWidget *bvw;
 	xine_cfg_entry_t entry;
@@ -1562,21 +1531,39 @@ bacon_video_widget_new (int width, int height,
 	bvw = BACON_VIDEO_WIDGET (g_object_new
 			(bacon_video_widget_get_type (), NULL));
 
-	bvw->priv->null_out = null_out;
-
 	bvw->priv->init_width = width;
 	bvw->priv->init_height = height;
+	bvw->priv->type = type;
+
+	/* Don't load anything yet if we're looking for proper video
+	 * output */
+	if (type == BVW_USE_TYPE_VIDEO)
+	{
+		bvw_config_helper_num (bvw->priv->xine, "video.num_buffers",
+				500, &entry);
+		entry.num_value = 500;
+		xine_config_update_entry (bvw->priv->xine, &entry);
+		return GTK_WIDGET (bvw);
+	}
 
 	/* load the output drivers */
-	bvw->priv->ao_driver = load_audio_out_driver (bvw, error);
-	if (error != NULL && *error != NULL)
-		return NULL;
+	if (type == BVW_USE_TYPE_AUDIO)
+	{
+		bvw->priv->ao_driver = load_audio_out_driver (bvw, error);
+		if (error != NULL && *error != NULL)
+			return NULL;
+	}
 
-	bvw->priv->vo_driver = load_video_out_driver (bvw, TRUE);
+	/* We need to wait for the widget to realise if we want to
+	 * load a video output with screen output, and capture is the
+	 * only one actually needing a video output */
+	if (type == BVW_USE_TYPE_CAPTURE) {
+		bvw->priv->vo_driver = load_video_out_driver (bvw, TRUE);
+	}
 
 	/* Be extra careful about exiting out nicely when a video output
 	 * isn't available */
-	if (bvw->priv->vo_driver == NULL)
+	if (type == BVW_USE_TYPE_CAPTURE && bvw->priv->vo_driver == NULL)
 	{
 		/* Close the xine stuff */
 		if (bvw->priv->ao_driver != NULL) {
@@ -1600,20 +1587,14 @@ bacon_video_widget_new (int width, int height,
 		return NULL;
 	}
 
-	if (bvw->priv->null_out != FALSE)
-	{
-		bvw_config_helper_num (bvw->priv->xine, "video.num_buffers",
-				5, &entry);
-		entry.num_value = 5;
-	} else {
-		bvw_config_helper_num (bvw->priv->xine, "video.num_buffers",
-				500, &entry);
-		entry.num_value = 500;
-	}
+	bvw_config_helper_num (bvw->priv->xine, "video.num_buffers",
+			5, &entry);
+	entry.num_value = 5;
 	xine_config_update_entry (bvw->priv->xine, &entry);
 
 	bvw->priv->stream = xine_stream_new (bvw->priv->xine,
 			bvw->priv->ao_driver, bvw->priv->vo_driver);
+	setup_config_stream (bvw);
 	bvw->priv->ev_queue = xine_event_new_queue (bvw->priv->stream);
 
 	return GTK_WIDGET (bvw);
@@ -1880,6 +1861,10 @@ bacon_video_widget_get_subtitle_mrl (const char *mrl)
 		return NULL;
 	}
 
+	if (strstr (mrl, "#subtitle:") != NULL) {
+		return NULL;
+	}
+
 	len = strlen (mrl);
 	if (mrl[len-4] != '.') {
 		return NULL;
@@ -1960,9 +1945,6 @@ bacon_video_widget_open (BaconVideoWidget *bvw, const char *mrl,
 		is_audio = (xine_get_stream_info (bvw->priv->stream,
 				XINE_STREAM_INFO_HAS_VIDEO) == FALSE);
 
-		g_signal_emit (G_OBJECT (bvw),
-				bvw_table_signals[GOT_METADATA], 0, NULL);
-
 		name = bacon_video_widget_get_nice_codec_name (bvw, is_audio);
 
 		bacon_video_widget_close (bvw);
@@ -1984,12 +1966,9 @@ bacon_video_widget_open (BaconVideoWidget *bvw, const char *mrl,
 
 	if (xine_get_stream_info (bvw->priv->stream,
 				XINE_STREAM_INFO_HAS_VIDEO) == FALSE
-		&& bvw->priv->ao_driver == NULL
-		&& bvw->priv->null_out != FALSE)
+		&& bvw->priv->type != BVW_USE_TYPE_METADATA
+		&& bvw->priv->ao_driver == NULL)
 	{
-		g_signal_emit (G_OBJECT (bvw),
-				bvw_table_signals[GOT_METADATA], 0, NULL);
-
 		bacon_video_widget_close (bvw);
 
 		g_set_error (error, BVW_ERROR, BVW_ERROR_AUDIO_ONLY,
@@ -2105,7 +2084,7 @@ gboolean bacon_video_widget_seek (BaconVideoWidget *bvw, float position,
 gboolean bacon_video_widget_seek_time (BaconVideoWidget *bvw, gint64 time,
 		GError **gerror)
 {
-	int error, speed;
+	int error, speed, status;
 	gint64 length;
 
 	g_return_val_if_fail (bvw != NULL, -1);
@@ -2115,7 +2094,8 @@ gboolean bacon_video_widget_seek_time (BaconVideoWidget *bvw, gint64 time,
 	length = bacon_video_widget_get_stream_length (bvw);
 
 	speed = xine_get_param (bvw->priv->stream, XINE_PARAM_SPEED);
-	if (speed == XINE_SPEED_PAUSE)
+	status = xine_get_status (bvw->priv->stream);
+	if (speed == XINE_SPEED_PAUSE || status == XINE_STATUS_STOP)
 	{
 		bvw->priv->seeking = 2;
 		bvw->priv->seek_dest_time = CLAMP (time, 0, length);
@@ -2370,18 +2350,6 @@ bacon_video_widget_get_volume (BaconVideoWidget *bvw)
 	if (bacon_video_widget_can_set_volume (bvw) == FALSE)
 		return 0;
 
-	if (bvw->priv->volume == -1)
-	{
-		xine_cfg_entry_t entry;
-
-		xine_config_lookup_entry (bvw->priv->xine,
-				"misc.amp_level", &entry);
-		bvw->priv->volume = entry.num_value;
-
-		xine_set_param (bvw->priv->stream,
-				XINE_PARAM_AUDIO_AMP_LEVEL, bvw->priv->volume);
-	}
-
 	return bvw->priv->volume;
 }
 
@@ -2427,9 +2395,9 @@ bacon_video_widget_fullscreen_mode_available (BaconVideoWidget *bvw,
 void
 bacon_video_widget_set_fullscreen (BaconVideoWidget *bvw, gboolean fullscreen)
 {
-	if (((bvw->priv->auto_resize == FALSE) || (bvw->priv->have_xrandr == FALSE)) &&
-			(bvw->priv->tvout != TV_OUT_NVTV_NTSC) &&
-			(bvw->priv->tvout != TV_OUT_NVTV_PAL))
+	if (bvw->priv->have_xvidmode == FALSE &&
+			bvw->priv->tvout != TV_OUT_NVTV_NTSC &&
+			bvw->priv->tvout != TV_OUT_NVTV_PAL)
 		return;
 
 	bvw->priv->fullscreen_mode = fullscreen;
@@ -2437,29 +2405,27 @@ bacon_video_widget_set_fullscreen (BaconVideoWidget *bvw, gboolean fullscreen)
 	if (fullscreen == FALSE)
 	{
 #ifdef HAVE_NVTV
-        /* If NVTV is used */
-        if (nvtv_simple_get_state() == NVTV_SIMPLE_TV_ON) {
-		nvtv_simple_switch(NVTV_SIMPLE_TV_OFF,0,0);
+		/* If NVTV is used */
+		if (nvtv_simple_get_state() == NVTV_SIMPLE_TV_ON) {
+			nvtv_simple_switch(NVTV_SIMPLE_TV_OFF,0,0);
         
-		/* Else if just auto resize is used */
-        } else if (bvw->priv->auto_resize != FALSE) {
+			/* Else if just auto resize is used */
+		} else if (bvw->priv->auto_resize != FALSE) {
 #endif
-		bacon_restore (bvw->priv->screenid);
+			bacon_restore ();
 #ifdef HAVE_NVTV
-	}
-	/* Turn fullscreen on with NVTV if that option is on */
+		}
+		/* Turn fullscreen on with NVTV if that option is on */
 	} else if ((bvw->priv->tvout == TV_OUT_NVTV_NTSC) ||
 			(bvw->priv->tvout == TV_OUT_NVTV_PAL)) {
 		nvtv_simple_switch(NVTV_SIMPLE_TV_ON,
 				bvw->priv->video_width,
 				bvw->priv->video_height);
 #endif
-		/* Turn fullscreen on with autoresize */
-	} else if (bvw->priv->auto_resize != FALSE) {
-		bvw->priv->screenid = bacon_resize_get_current ();
-		bacon_resize (bvw->priv->video_height,
-				bvw->priv->video_width);
-	} 
+		/* Turn fullscreen on when we have xvidmode */
+	} else if (bvw->priv->have_xvidmode != FALSE) {
+		bacon_resize ();
+	}
 }
 
 void
@@ -2710,7 +2676,7 @@ bacon_video_widget_set_visuals (BaconVideoWidget *bvw, const char *name)
 	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
 	g_return_val_if_fail (bvw->priv->xine != NULL, FALSE);
 
-	if (bvw->priv->null_out != FALSE || bvw->priv->ao_driver == NULL)
+	if (bvw->priv->type != BVW_USE_TYPE_VIDEO)
 		return FALSE;
 
 	if (GTK_WIDGET_REALIZED (bvw) == FALSE)
@@ -2841,13 +2807,15 @@ bacon_video_widget_get_current_time (BaconVideoWidget *bvw)
 {
 	int pos_time = 0, i = 0;
 	int pos_stream, length_time;
+	int status;
 	gboolean ret;
 
 	g_return_val_if_fail (bvw != NULL, 0);
 	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), 0);
 	g_return_val_if_fail (bvw->priv->xine != NULL, 0);
 
-	if (bacon_video_widget_is_playing (bvw) == FALSE)
+	status = xine_get_status (bvw->priv->stream);
+	if (status != XINE_STATUS_STOP && status != XINE_STATUS_PLAY)
 		return 0;
 
 	ret = xine_get_pos_length (bvw->priv->stream, &pos_stream,
@@ -3059,6 +3027,35 @@ bacon_video_widget_set_scale_ratio (BaconVideoWidget *bvw, gfloat ratio)
 	totem_widget_set_preferred_size (toplevel, new_w, new_h);
 }
 
+gboolean
+bacon_video_widget_can_set_zoom (BaconVideoWidget *bvw)
+{
+	return TRUE;
+}
+
+void
+bacon_video_widget_set_zoom (BaconVideoWidget *bvw, int zoom)
+{
+	g_return_if_fail (bvw != NULL);
+	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
+	g_return_if_fail (bvw->priv->xine != NULL);
+
+	xine_set_param (bvw->priv->stream,
+			XINE_PARAM_VO_ZOOM_X, zoom);
+	xine_set_param (bvw->priv->stream,
+			XINE_PARAM_VO_ZOOM_Y, zoom);
+}
+
+int
+bacon_video_widget_get_zoom (BaconVideoWidget *bvw)
+{
+	g_return_val_if_fail (bvw != NULL, 100);
+	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), 100);
+	g_return_val_if_fail (bvw->priv->xine != NULL, 100);
+
+	return xine_get_param (bvw->priv->stream, XINE_PARAM_VO_ZOOM_X);
+}
+
 int
 bacon_video_widget_get_video_property (BaconVideoWidget *bvw,
 		BaconVideoWidgetVideoProperty type)
@@ -3126,6 +3123,9 @@ bacon_video_widget_set_audio_out_type (BaconVideoWidget *bvw,
 	case BVW_AUDIO_SOUND_4CHANNEL:
 		value = 6;
 		break;
+	case BVW_AUDIO_SOUND_41CHANNEL:
+		value = 7;
+		break;
 	case BVW_AUDIO_SOUND_5CHANNEL:
 		value = 8;
 		break;
@@ -3136,7 +3136,7 @@ bacon_video_widget_set_audio_out_type (BaconVideoWidget *bvw,
 		value = 3;
 		break;
 	default:
-		value = 2;
+		value = 1;
 		g_warning ("Unsupported audio type %d selected", type);
 	}
 
@@ -3173,6 +3173,7 @@ bacon_video_widget_get_metadata_string (BaconVideoWidget *bvw, BaconVideoWidgetM
 	case BVW_INFO_ALBUM:
 		string = xine_get_meta_info (bvw->priv->stream,
 				XINE_META_INFO_ALBUM);
+		break;
 	case BVW_INFO_YEAR:
 		string = xine_get_meta_info (bvw->priv->stream,
 				XINE_META_INFO_YEAR);
@@ -3202,6 +3203,7 @@ bacon_video_widget_get_metadata_string (BaconVideoWidget *bvw, BaconVideoWidgetM
 		{
 			char *utf8;
 
+			g_warning ("Metadata for index %d not in UTF-8 for mrl '%s'", type, bvw->priv->mrl);
 			utf8 = g_locale_to_utf8 (string, -1, NULL, NULL, NULL);
 			g_value_set_string (value, utf8);
 			g_free (utf8);
@@ -3315,6 +3317,7 @@ bacon_video_widget_get_metadata (BaconVideoWidget *bvw,
 	case BVW_INFO_YEAR:
 	case BVW_INFO_VIDEO_CODEC:
 	case BVW_INFO_AUDIO_CODEC:
+	case BVW_INFO_CDINDEX:
 		bacon_video_widget_get_metadata_string (bvw, type, value);
 		break;
 	case BVW_INFO_DURATION:
@@ -3335,27 +3338,6 @@ bacon_video_widget_get_metadata (BaconVideoWidget *bvw,
 	return;
 }
 
-char
-*bacon_video_widget_properties_get_title (BaconVideoWidget *bvw)
-{
-	const char *short_title, *artist;
-
-	artist = xine_get_meta_info (bvw->priv->stream, XINE_META_INFO_ARTIST);
-	short_title = xine_get_meta_info (bvw->priv->stream,
-			XINE_META_INFO_TITLE);
-
-	if (artist == NULL && short_title == NULL)
-		return NULL;
-
-	if (artist == NULL && short_title != NULL)
-		return g_strdup (short_title);
-
-	if (artist != NULL && short_title != NULL)
-		return g_strdup_printf ("%s - %s", artist, short_title);
-
-	return NULL;
-}
-
 GList
 *bacon_video_widget_get_languages (BaconVideoWidget *bvw)
 {
@@ -3408,7 +3390,7 @@ GList
 	{
 		memset (&lang, 0, sizeof (lang));
 
-		if (xine_get_spu_lang(bvw->priv->stream, i, lang) == 1)
+		if (xine_get_spu_lang (bvw->priv->stream, i, lang) == 1)
 		{
 			list = g_list_prepend (list,
 					(gpointer) g_strdup (lang));
