@@ -22,7 +22,10 @@
    Author: Bastien Nocera <hadess@hadess.net>
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include "totem-pl-parser.h"
 
 #include "totemplparser-marshal.h"
@@ -33,8 +36,6 @@
 #include <gtk/gtk.h>
 #include <libxml/tree.h>
 #include <libxml/parser.h>
-#include <glade/glade.h>
-#include <gconf/gconf-client.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
@@ -934,9 +935,34 @@ totem_pl_parser_add_pls (TotemPlParser *parser, const char *url, gpointer data)
 		g_free (title_key);
 		g_free (genre_key);
 
-		if (file != NULL)
+		if (file == NULL)
+		{
+			g_free (file);
+			g_free (title);
+			g_free (genre);
+			continue;
+		}
+
+		if (strstr (file, "://") != NULL
+				|| file[0] == G_DIR_SEPARATOR) {
 			totem_pl_parser_add_one_url_ext (parser,
 					file, title, genre);
+		} else {
+			char *uri, *base, *escaped;
+
+			/* Try with a base */
+			base = totem_pl_parser_base_url (url);
+			escaped = gnome_vfs_escape_path_string (file);
+
+			uri = g_strdup_printf ("%s/%s", base, escaped);
+
+			totem_pl_parser_add_one_url_ext (parser,
+					uri, title, genre);
+
+			g_free (escaped);
+			g_free (uri);
+			g_free (base);
+		}
 
 		g_free (file);
 		g_free (title);
@@ -1291,7 +1317,7 @@ totem_pl_parser_add_directory (TotemPlParser *parser, const char *url,
 	char *media_url;
 
 	if (parser->priv->recurse_level == 1) {
-		type = cd_detect_type_from_dir (url, &media_url, NULL);
+		type = totem_cd_detect_type_from_dir (url, &media_url, NULL);
 		if (type != MEDIA_TYPE_DATA && type != MEDIA_TYPE_ERROR) {
 			if (media_url != NULL) {
 				totem_pl_parser_add_one_url (parser, media_url, NULL);
@@ -1422,8 +1448,6 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 	if (parser->priv->recurse_level > RECURSE_LEVEL_MAX)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
-	parser->priv->recurse_level++;
-
 	if (totem_pl_parser_ignore (parser, url) != FALSE)
 		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
@@ -1434,6 +1458,8 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 	if (mimetype == NULL)
 		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
+	parser->priv->recurse_level++;
+
 	for (i = 0; i < G_N_ELEMENTS(special_types); i++) {
 		if (strcmp (special_types[i].mimetype, mimetype) == 0) {
 			ret = (* special_types[i].func) (parser, url, data);
@@ -1453,6 +1479,8 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 		}
 	}
 
+	parser->priv->recurse_level--;
+
 	if (ret == FALSE && parser->priv->fallback) {
 		totem_pl_parser_add_one_url (parser, url, NULL);
 		return TOTEM_PL_PARSER_RESULT_SUCCESS;
