@@ -105,6 +105,7 @@ static int totem_playlist_table_signals[LAST_SIGNAL] = { 0 };
 
 static const GtkTargetEntry target_table[] = {
 	{ "text/uri-list", 0, 0 },
+	{ "_NETSCAPE_URL", 0, 1 },
 };
 
 static GtkWidgetClass *parent_class = NULL;
@@ -380,7 +381,22 @@ drop_cb (GtkWidget     *widget,
 					| G_FILE_TEST_EXISTS)
 				 || strstr (filename, "://") != NULL))
 		{
-			totem_playlist_add_mrl (playlist, filename, NULL);
+			char *title = NULL;
+
+			/* Super _NETSCAPE_URL trick */
+			if (info == 1)
+			{
+				g_free (p->data);
+				p = p->next;
+				if (p != NULL) {
+					if (g_str_has_prefix (p->data, "file:") != FALSE)
+						title = (char *) p->data + 5;
+					else
+						title = p->data;
+				}
+			}
+
+			totem_playlist_add_mrl (playlist, filename, title);
 		}
 		g_free (filename);
 		g_free (p->data);
@@ -456,14 +472,59 @@ treeview_button_pressed (GtkTreeView *treeview, GdkEventButton *event,
 	return TRUE;
 }
 
+static void
+totem_playlist_set_reorderable (TotemPlaylist *playlist, gboolean set)
+{
+	guint num_items, i;
+
+	gtk_tree_view_set_reorderable
+		(GTK_TREE_VIEW (playlist->_priv->treeview), set);
+
+	if (set != FALSE)
+		return;
+
+	num_items = PL_LEN;
+	for (i = 0; i < num_items; i++)
+	{
+		GtkTreeIter iter;
+		char *index;
+		GdkPixbuf *pixbuf;
+
+		index = g_strdup_printf ("%d", i);
+		if (gtk_tree_model_get_iter_from_string
+				(playlist->_priv->model,
+				 &iter, index) == FALSE)
+		{
+			g_free (index);
+			continue;
+		}
+		g_free (index);
+
+		gtk_tree_model_get (playlist->_priv->model, &iter,
+				PIX_COL, &pixbuf, -1);
+
+		if (pixbuf == NULL)
+			continue;
+
+		gdk_pixbuf_unref (pixbuf);
+		gtk_tree_path_free (playlist->_priv->current);
+		playlist->_priv->current = gtk_tree_path_new_from_indices (i, -1);
+		g_signal_emit (G_OBJECT (playlist),
+				totem_playlist_table_signals[CHANGED],
+				0, NULL);
+
+		break;
+	}
+}
+
 static gboolean 
 button_press_cb (GtkWidget *treeview, GdkEventButton *event, gpointer data)
 { 
-	
 	gtk_drag_dest_unset (treeview);
 	g_signal_handlers_block_by_func (treeview, (GFunc) drop_cb, data);
-	gtk_tree_view_set_reorderable (GTK_TREE_VIEW (treeview), TRUE);
-	
+
+	totem_playlist_set_reorderable ((TotemPlaylist *) data, TRUE);
+
 	return FALSE;
 }
 
@@ -474,15 +535,13 @@ button_release_cb (GtkWidget *treeview, GdkEventButton *event, gpointer data)
 
 	if (!playlist->_priv->drag_started)
 	{
-		gtk_tree_view_set_reorderable (GTK_TREE_VIEW (treeview),
-				FALSE);
+		totem_playlist_set_reorderable (playlist, FALSE);
 		gtk_drag_dest_set (treeview, GTK_DEST_DEFAULT_ALL,
 				target_table, G_N_ELEMENTS (target_table),
 				GDK_ACTION_COPY);
 
 		g_signal_handlers_unblock_by_func (treeview,
 				(GFunc) drop_cb, data);
-		
 	}
 
 	return FALSE;
@@ -504,7 +563,7 @@ drag_end_cb (GtkWidget *treeview, GdkDragContext *context, gpointer data)
 	TotemPlaylist *playlist = (TotemPlaylist *)data;
 
 	playlist->_priv->drag_started = FALSE;
-	gtk_tree_view_set_reorderable (GTK_TREE_VIEW (treeview), FALSE); 
+	totem_playlist_set_reorderable (playlist, FALSE);
 	gtk_drag_dest_set (treeview, GTK_DEST_DEFAULT_ALL, target_table,
         		G_N_ELEMENTS (target_table), GDK_ACTION_COPY);
 
@@ -582,7 +641,7 @@ totem_playlist_add_files (GtkWidget *widget, TotemPlaylist *playlist)
 
 	if (playlist->_priv->path != NULL)
 	{
-		gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (fs),
+		gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (fs),
 				playlist->_priv->path);
 	}
 	response = gtk_dialog_run (GTK_DIALOG (fs));
@@ -605,13 +664,8 @@ totem_playlist_add_files (GtkWidget *widget, TotemPlaylist *playlist)
 		mrl = filenames->data;
 		if (mrl != NULL)
 		{
-			char *tmp;
-
-			tmp = g_path_get_dirname (mrl);
 			g_free (playlist->_priv->path);
-			playlist->_priv->path = g_strconcat (tmp,
-					G_DIR_SEPARATOR_S, NULL);
-			g_free (tmp);
+			playlist->_priv->path = g_path_get_dirname (mrl);
 		}
 
 		for (l = filenames; l != NULL; l = l->next)
@@ -762,7 +816,7 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 
 	if (playlist->_priv->save_path != NULL)
 	{
-		gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (fs),
+		gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (fs),
 				playlist->_priv->save_path);
 	}
 
@@ -773,7 +827,7 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 
 	if (response == GTK_RESPONSE_ACCEPT)
 	{
-		char *filename, *tmp;
+		char *filename;
 		GnomeVFSResult res;
 		GnomeVFSFileInfo info;
 
@@ -784,11 +838,8 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 		if (filename == NULL)
 			return;
 
-		tmp = g_path_get_dirname (filename);
 		g_free (playlist->_priv->save_path);
-		playlist->_priv->save_path = g_strconcat (tmp,
-				G_DIR_SEPARATOR_S, NULL);
-		g_free (tmp);
+		playlist->_priv->save_path = g_path_get_dirname (filename);
 
 		res = gnome_vfs_get_file_info (filename, &info, GNOME_VFS_FILE_INFO_DEFAULT);
 		if (res != GNOME_VFS_ERROR_NOT_FOUND && res == GNOME_VFS_OK)
@@ -953,6 +1004,7 @@ totem_playlist_down_files (GtkWidget *widget, TotemPlaylist *playlist)
 static int
 totem_playlist_key_press (GtkWidget *win, GdkEventKey *event, TotemPlaylist *playlist)
 {
+	//FIXME add support for GDK_Menu and Ctrl+F10
 	/* Special case some shortcuts */
 	if (event->state != 0
 			&& (event->state &GDK_CONTROL_MASK))
@@ -977,6 +1029,9 @@ totem_playlist_key_press (GtkWidget *win, GdkEventKey *event, TotemPlaylist *pla
 	{
 		totem_playlist_remove_files (NULL, playlist);
 		return TRUE;
+	} else if (event->keyval == GDK_Escape) {
+		gtk_dialog_response (GTK_DIALOG (playlist),
+				GTK_RESPONSE_CLOSE);
 	}
 
 	return FALSE;
@@ -1010,7 +1065,12 @@ treeview_row_changed (GtkTreeView *treeview, GtkTreePath *arg1,
 {
 	if (totem_playlist_gtk_tree_path_equals
 			(arg1, playlist->_priv->current) != FALSE)
+	{
+		g_signal_emit (G_OBJECT (playlist),
+				totem_playlist_table_signals[CHANGED], 0,
+				NULL);
 		return;
+	}
 
 	if (playlist->_priv->current != NULL)
 	{
@@ -1039,6 +1099,9 @@ treeview_row_changed (GtkTreeView *treeview, GtkTreePath *arg1,
 	g_signal_emit (G_OBJECT (playlist),
 			totem_playlist_table_signals[CHANGED], 0,
 			NULL);
+
+	g_signal_handlers_unblock_by_func (treeview,
+			(GFunc) drop_cb, playlist);
 }
 
 static void
@@ -1082,7 +1145,8 @@ init_treeview (GtkWidget *treeview, TotemPlaylist *playlist)
          g_signal_connect (G_OBJECT (treeview), "drag_end",
                         G_CALLBACK (drag_end_cb), playlist);
 	gtk_drag_dest_set (treeview, GTK_DEST_DEFAULT_ALL,
-			target_table, 1, GDK_ACTION_COPY);
+			target_table, G_N_ELEMENTS (target_table),
+			GDK_ACTION_COPY);
 
 	playlist->_priv->selection = selection;
 
@@ -1979,6 +2043,46 @@ totem_playlist_set_at_end (TotemPlaylist *playlist)
 	}
 }
 
+guint
+totem_playlist_get_current (TotemPlaylist *playlist)
+{
+	char *path;
+	double index;
+
+	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), 0);
+
+	path = gtk_tree_path_to_string (playlist->_priv->current);
+	if (path == NULL)
+		return -1;
+
+	index = g_ascii_strtod (path, NULL);
+	g_free (path);
+
+	return index;
+}
+
+guint
+totem_playlist_get_last (TotemPlaylist *playlist)
+{
+	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), -1);
+
+	return PL_LEN - 1;
+}
+
+void
+totem_playlist_set_current (TotemPlaylist *playlist, guint index)
+{
+	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
+
+	if (index >= (guint) PL_LEN)
+		return;
+
+	totem_playlist_unset_playing (playlist);
+	//FIXME problems when shuffled?
+	gtk_tree_path_free (playlist->_priv->current);
+	playlist->_priv->current = gtk_tree_path_new_from_indices (index, -1);
+}
+
 static void
 totem_playlist_class_init (TotemPlaylistClass *klass)
 {
