@@ -31,9 +31,10 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtkvseparator.h>
 #include <gtk/gtkwindow.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 
 #include "totem-statusbar.h"
+#include "video-utils.h"
 
 static void     totem_statusbar_class_init     (TotemStatusbarClass *class);
 static void     totem_statusbar_init           (TotemStatusbar      *statusbar);
@@ -57,55 +58,7 @@ static void     totem_statusbar_destroy_window (TotemStatusbar      *statusbar);
 
 static GtkContainerClass *parent_class;
 
-static char
-*time_to_string (int time)
-{
-  int sec, min, hour;
-                                                                              
-  sec = time % 60;
-  time = time - sec;
-  min = (time % (60*60)) / 60;
-  time = time - (min * 60);
-  hour = time / (60*60);
-
-  if (hour > 0)
-  {
-    /* hour:minutes:seconds */
-    return g_strdup_printf ("%d:%02d:%02d", hour, min, sec);
-  } else {
-    /* minutes:seconds */
-    return g_strdup_printf ("%d:%02d", min, sec);
-  }
-
-  return NULL;
-}
-
-GType      
-totem_statusbar_get_type (void)
-{
-  static GType statusbar_type = 0;
-
-  if (!statusbar_type)
-    {
-      static const GTypeInfo statusbar_info =
-      {
-        sizeof (TotemStatusbarClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-        (GClassInitFunc) totem_statusbar_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-        sizeof (TotemStatusbar),
-	0,		/* n_preallocs */
-        (GInstanceInitFunc) totem_statusbar_init,
-      };
-
-      statusbar_type = g_type_register_static (GTK_TYPE_HBOX, "TotemStatusbar",
-					       &statusbar_info, 0);
-    }
-
-  return statusbar_type;
-}
+G_DEFINE_TYPE(TotemStatusbar, totem_statusbar, GTK_TYPE_HBOX)
 
 static void
 totem_statusbar_class_init (TotemStatusbarClass *class)
@@ -169,16 +122,24 @@ totem_statusbar_init (TotemStatusbar *statusbar)
   gtk_widget_show (hbox);
 
   statusbar->label = gtk_label_new (_("Stopped"));
-  gtk_misc_set_alignment (GTK_MISC (statusbar->label), 0.0, 0.0);
+  gtk_misc_set_alignment (GTK_MISC (statusbar->label), 0.0, 0.5);
   gtk_box_pack_start (GTK_BOX (hbox), statusbar->label, FALSE, FALSE, 0);
   gtk_widget_show (statusbar->label);
 
+  /* progressbar for network streams */
+  statusbar->progress = gtk_progress_bar_new ();
+  gtk_progress_bar_set_orientation (GTK_PROGRESS_BAR (statusbar->progress),
+				    GTK_ORIENTATION_HORIZONTAL);
+  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (statusbar->progress), 0.);
+  gtk_box_pack_start (GTK_BOX (hbox), statusbar->progress, FALSE, FALSE, 0);
+  gtk_widget_hide (statusbar->progress);
+
   packer = gtk_vseparator_new ();
   gtk_box_pack_start (GTK_BOX (hbox), packer, FALSE, FALSE, 0);
   gtk_widget_show (packer);
 
   statusbar->time_label = gtk_label_new (_("0:00 / 0:00"));
-  gtk_misc_set_alignment (GTK_MISC (statusbar->label), 0.0, 0.0);
+  gtk_misc_set_alignment (GTK_MISC (statusbar->label), 0.0, 0.5);
   gtk_box_pack_start (GTK_BOX (hbox), statusbar->time_label, FALSE, FALSE, 0);
   gtk_widget_show (statusbar->time_label);
 
@@ -199,18 +160,21 @@ totem_statusbar_update_time (TotemStatusbar *statusbar)
 {
   char *time, *length, *label;
 
-  if (statusbar->pushed == 1)
-    return;
-
-  time = time_to_string (statusbar->time);
+  time = totem_time_to_string (statusbar->time * 1000);
 
   if (statusbar->length < 0) {
     label = g_strdup_printf (_("%s (Streaming)"), time);
   } else {
-    length = time_to_string (statusbar->length == -1 ? 0 : statusbar->length);
+    length = totem_time_to_string
+	    (statusbar->length == -1 ? 0 : statusbar->length * 1000);
+
+    if (statusbar->seeking == FALSE)
+      /* Elapsed / Total Length */
+      label = g_strdup_printf (_("%s / %s"), time, length);
+    else
+      /* Seeking to Time / Total Length */
+      label = g_strdup_printf (_("Seek to %s / %s"), time, length);
 
-    //FIXME LTR/RTL
-    label = g_strdup_printf ("%s / %s", time, length);
     g_free (length);
   }
 
@@ -243,7 +207,7 @@ totem_statusbar_timeout_pop (TotemStatusbar *statusbar)
   g_free (statusbar->saved_label);
   statusbar->saved_label = NULL;
   statusbar->pushed = 0;
-  totem_statusbar_update_time (statusbar);
+  gtk_widget_hide (statusbar->progress);
 
   return FALSE;
 }
@@ -259,12 +223,6 @@ totem_statusbar_push (TotemStatusbar *statusbar, guint percentage)
     g_source_remove (statusbar->timeout);
   }
 
-  if (percentage == 100)
-  {
-    totem_statusbar_timeout_pop (statusbar);
-    return;
-  }
-
   if (statusbar->saved_label == NULL)
   {
     statusbar->saved_label = g_strdup
@@ -275,8 +233,11 @@ totem_statusbar_push (TotemStatusbar *statusbar, guint percentage)
 
   /* eg: 75 % */
   label = g_strdup_printf (_("%d %%"), percentage);
-  gtk_label_set_text (GTK_LABEL (statusbar->time_label), label);
+  gtk_progress_bar_set_text (GTK_PROGRESS_BAR (statusbar->progress), label);
   g_free (label);
+  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (statusbar->progress),
+				 percentage / 100.);
+  gtk_widget_show (statusbar->progress);
 
   statusbar->timeout = g_timeout_add (3000,
 		  (GSourceFunc) totem_statusbar_timeout_pop, statusbar);
@@ -294,8 +255,22 @@ totem_statusbar_set_time_and_length (TotemStatusbar *statusbar,
 {
   g_return_if_fail (TOTEM_IS_STATUSBAR (statusbar));
 
-  statusbar->time = time;
-  statusbar->length = length;
+  if (time != statusbar->time ||
+      length != statusbar->length) {
+    statusbar->time = time;
+    statusbar->length = length;
+
+    totem_statusbar_update_time (statusbar);
+  }
+}
+
+void
+totem_statusbar_set_seeking (TotemStatusbar *statusbar,
+			     gboolean seeking)
+{
+  g_return_if_fail (TOTEM_IS_STATUSBAR (statusbar));
+
+  statusbar->seeking = seeking;
 
   totem_statusbar_update_time (statusbar);
 }
